package com.bomostory.sceneeditmodule

import android.content.DialogInterface
import android.media.MediaPlayer
import android.support.v7.app.AppCompatActivity
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.support.v4.app.DialogFragment
import android.view.View
import android.widget.CompoundButton
import android.widget.SeekBar
import com.bomostory.sceneeditmodule.basicdata.DataParser
import com.bomostory.sceneeditmodule.basicdata.Music
import com.bomostory.sceneeditmodule.basicdata.Scene
import com.bomostory.sceneeditmodule.basicdata.Story
import com.bomostory.sceneeditmodule.view.AudioTrackGroupView
import com.bomostory.sceneeditmodule.view.MovieSelectView
import com.example.tfat.myapplication.R
import com.google.gson.Gson
import io.reactivex.Observable
import io.reactivex.Observer
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_movie_edit.*
import java.util.concurrent.TimeUnit
import java.util.concurrent.atomic.AtomicLong

class MovieEditActivity : AppCompatActivity(), DialogInterface.OnDismissListener, MusicSelectFragment.OnFragmentInteractionListener {
    override fun onDismiss(dialog: DialogInterface?) {
        mediaPlayer.apply {
            stop()
        }

        delayedHide(100)
    }

    override fun onMusicPlayClick(path: String) {
        mediaPlayer.apply {
            reset()
            setDataSource(path)
            prepareAsync()
        }
    }

    override fun onMusicPauseClick() {
        mediaPlayer.apply {
            reset()
        }
    }

    override fun onMusicImportClick(music: Music) {
        musics.add(music)

        movieEditView.musics = musics

        val dialogFragment = supportFragmentManager.findFragmentByTag(MUSIC_SELECT_DIALOG_TAG) as DialogFragment
        dialogFragment.dismiss()
    }

    private val mHideHandler = Handler()
    private val mHidePart2Runnable = Runnable {
        window.decorView.systemUiVisibility =
                View.SYSTEM_UI_FLAG_LOW_PROFILE or
                View.SYSTEM_UI_FLAG_FULLSCREEN or
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE or
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY or
                View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or
                View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
    }
    private val mShowPart2Runnable = Runnable {
        supportActionBar?.show()
    }
    private val mHideRunnable = Runnable {
        hide()
    }

    private var story: Story? = null
        set(value) {
            value?.let {
                movieEditView.story = value
                scene = value.scenes[0]
            }
        }

    private var scene: Scene? = null
        set(value) {
            field = value
            value?.let {
                movieView.scene = value
                movieEditView.scene = value
            }
        }

    private var isLoop = false
        set(value) {
            field = value
            movieEditView.isLoop = value
        }

    private var currentAudioPosition = 0
        set(value) {
            field = value
            movieEditView.currentAudioPosition = value
        }

    private var disposable: Disposable? = null

    private val lastTick = AtomicLong(0L)

    private val mediaPlayer = MediaPlayer()

    private val musics = ArrayList<Music>()

    companion object {
        const val STORY_KEY = "story_key"
        const val MUSIC_SELECT_DIALOG_TAG = "music_select_dialog_tag"
        const val INTERVAL_PERIOD: Long = 1
        private const val UI_ANIMATION_DELAY = 300
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_movie_edit)

        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        story = Gson().fromJson<Story>(intent.extras.getString(STORY_KEY), Story::class.java)

        movieEditView.onPlayClickListener = View.OnClickListener(this::playMovie)
        movieEditView.onPauseClickListener = View.OnClickListener(this::pauseMovie)
        movieEditView.onAddMusicClickListener = View.OnClickListener(this::addMusic)
        movieEditView.onDeleteMusicClickListener = View.OnClickListener(this::deleteMusic)
        movieEditView.onSeekBarChangeListener = object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                lastTick.set(progress.toLong())
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
            }
        }
        movieEditView.onMovieSelectListener = object : MovieSelectView.OnMovieSelectListener {
            override fun onMoveSelect(scene: Scene) {
                this@MovieEditActivity.scene = scene
            }
        }
        movieEditView.onAudioTrackSelectListener = object : AudioTrackGroupView.OnAudioTrackSelectListener {
            override fun onAudioTrackSelect(position: Int) {
                currentAudioPosition = position
            }
        }
        movieEditView.onLoopSwitchChangedListener = CompoundButton.OnCheckedChangeListener { _, isChecked ->
            isLoop = isChecked
        }

        mediaPlayer.setOnPreparedListener {
            it.start()
        }
    }

    override fun onPostCreate(savedInstanceState: Bundle?) {
        super.onPostCreate(savedInstanceState)
        delayedHide(100)
    }

    override fun onStop() {
        super.onStop()

        mediaPlayer.apply {
            release()
        }
    }

    private fun delayedHide(delayMillis: Int) {
        mHideHandler.removeCallbacks(mHideRunnable)
        mHideHandler.postDelayed(mHideRunnable, delayMillis.toLong())
    }

    private fun hide() {
        supportActionBar?.hide()
        mHideHandler.removeCallbacks(mShowPart2Runnable)
        mHideHandler.postDelayed(mHidePart2Runnable, UI_ANIMATION_DELAY.toLong())
    }

    private fun playMovie(view: View) {
        Observable.interval(INTERVAL_PERIOD, TimeUnit.MILLISECONDS)
                .map {
                    lastTick.getAndIncrement()
                }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : Observer<Long> {
                    override fun onComplete() {
                        movieEditView.playTime = 0
                        lastTick.set(0)
                    }

                    override fun onError(e: Throwable) {
                    }

                    override fun onSubscribe(d: Disposable) {
                        disposable = d
                    }

                    override fun onNext(t: Long) {
                        scene?.let {
                            it.record?.let {

                                for (track in it.tracks) {
                                    if (track.time == t) {
                                        movieView.x = track.positionX
                                    }
                                }

                                movieEditView.playTime = t

                                if (t > (it.period / INTERVAL_PERIOD)) {
                                    disposable?.dispose()
                                    onComplete()
                                }
                            }
                        }
                    }
                })
    }

    private fun pauseMovie(view: View) {
        disposable?.dispose()
    }

    private fun addMusic(view: View) {
        val musicSource = DataParser.parseMusicSource(contentResolver, (Environment.getExternalStorageDirectory().toString()
                + "/Android/yuheng/public/music"))
        val musicSelectDialog = MusicSelectDialog.newInstance(Gson().toJson(musicSource))
        musicSelectDialog.show(supportFragmentManager, MUSIC_SELECT_DIALOG_TAG)
    }

    private fun deleteMusic(view: View) {
        if (currentAudioPosition in 0 until musics.size) {
            musics.removeAt(currentAudioPosition)
            movieEditView.musics = musics
        }
    }
}